<template>
  <div class="app-container">
    <div class="search_form">
      <div>
        <span class="search_title">供应商名称：</span>
        <el-input v-model="searchForm.supplierName" style="width: 240px" placeholder="请输入" @change="handleQuery"
          clearable prefix-icon="Search" />
        <el-button type="primary" @click="handleQuery" style="margin-left: 10px">搜索</el-button>
      </div>
      <div>
        <el-button type="primary" @click="openForm('add')">新增入库</el-button>
        <el-button @click="handleOut">导出</el-button>
        <el-button type="danger" plain @click="handleDelete">删除</el-button>
      </div>
    </div>
    <div class="table_list">
      <el-table :data="tableData" border v-loading="tableLoading" @selection-change="handleSelectionChange"
        :expand-row-keys="expandedRowKeys" :row-key="row => row.id" show-summary style="width: 100%"
        :summary-method="summarizeMainTable" height="calc(100vh - 18.5em)">
        <el-table-column align="center" type="selection" width="55" />
        <el-table-column align="center" label="序号" type="index" width="60" />
        <el-table-column label="入库时间" prop="inboundTime" width="100" show-overflow-tooltip />
        <el-table-column label="入库批次" prop="inboundBatch" width="160" show-overflow-tooltip />
        <el-table-column label="供应商名称" prop="supplierName" width="160" show-overflow-tooltip />
        <el-table-column label="产品大类" prop="productName" width="100" show-overflow-tooltip />
        <el-table-column label="规格型号" prop="model" width="160" show-overflow-tooltip />
        <el-table-column label="单位" prop="unit" width="70" show-overflow-tooltip />
        <el-table-column label="入库数量" prop="inboundQuantity" width="90" show-overflow-tooltip />
        <el-table-column label="含税单价" prop="taxInclusiveUnitPrice" width="100" show-overflow-tooltip />
        <el-table-column label="含税总价" prop="taxInclusiveTotalPrice" width="100" show-overflow-tooltip />
        <el-table-column label="税率(%)" prop="taxRate" width="80" show-overflow-tooltip />
        <el-table-column label="不含税总价" prop="taxExclusiveTotalPrice" width="100" show-overflow-tooltip />
        <el-table-column label="入库人" prop="nickName" width="80" show-overflow-tooltip />
        <el-table-column fixed="right" label="操作" min-width="60" align="center">
          <template #default="scope">
            <el-button link type="primary" size="small" @click="openForm('edit', scope.row);">编辑</el-button>
          </template>
        </el-table-column>
      </el-table>
      <pagination v-show="total > 0" :total="total" layout="total, sizes, prev, pager, next, jumper"
        :page="page.current" :limit="page.size" @pagination="paginationChange" />
    </div>
    <el-dialog v-model="dialogFormVisible" :title="operationType === 'add' ? '新增入库' : '编辑入库'" width="70%"
      @close="closeDia">
      <el-form :model="form" label-width="140px" label-position="top" :rules="rules" ref="formRef">
        <el-form-item label="采购订单号" prop="salesContractNo">
              <el-input v-model="form.salesContractNo" placeholder="请输入" clearable />
            </el-form-item>
        <el-table
          :data="productData"
          border
          @selection-change="productSelected"
        >
          <el-table-column align="center" type="selection" width="55" />
          <el-table-column
            align="center"
            label="序号"
            type="index"
            width="60"
          />
          <el-table-column label="产品大类" prop="productCategory" />
          <el-table-column label="规格型号" prop="specificationModel" />
          <el-table-column label="单位" prop="unit" width="70" />
          <el-table-column label="采购数量" prop="quantity" width="100" />
          <el-table-column label="待入库数量" prop="quantity0" width="100" />
          <el-table-column label="本次入库数量" prop="quantityStock" width="120">
            <template #default="scope">
              <el-input v-model="scope.row.quantityStock" type="number" :min="0" :max="scope.row.quantity0" />
            </template>
          </el-table-column>
          <el-table-column label="税率(%)" prop="taxRate" width="120" />
          <el-table-column
            label="含税单价(元)"
            prop="taxInclusiveUnitPrice"
            :formatter="formattedNumber"
            width="150"
          />
          <el-table-column
            label="含税总价(元)"
            prop="taxInclusiveTotalPrice"
            :formatter="formattedNumber"
            width="150"
          />
          <el-table-column
            label="不含税总价(元)"
            prop="taxExclusiveTotalPrice"
            :formatter="formattedNumber"
            width="150"
          />
        </el-table>
      </el-form>
      <template #footer>
        <div class="dialog-footer">
          <el-button type="primary" @click="submitForm">确认</el-button>
          <el-button @click="closeDia">取消</el-button>
        </div>
      </template>
    </el-dialog>
  </div>
</template>

<script setup>
import pagination from '@/components/PIMTable/Pagination.vue'
import { ref } from 'vue'
import { ElMessageBox } from "element-plus";
import useUserStore from '@/store/modules/user'
import { userListNoPage } from "@/api/system/user.js";
import {
  getStockInPage,
  updateStockIn,
  addSutockIn,
  delStockIn,
  exportStockIn
} from "@/api/inventoryManagement/stockIn.js";
import {listSupplier} from "@/api/basicData/supplierManageFile.js";
import {modelListPage, productTreeList} from "@/api/basicData/product.js";
const userStore = useUserStore()
const { proxy } = getCurrentInstance()
const tableData = ref([])
const selectedRows = ref([])
const userList = ref([])
const supplierLists = ref([])
const productList = ref([]);
const productModelList = ref([]);
const purchaseRecords  = ref([]);
const loadingProducts = ref(false);

const loading = ref(false);
const tableLoading = ref(false)
const page = reactive({
  current: 1,
  size: 100,
})
const total = ref(0)

// 用户信息表单弹框数据
const operationType = ref('')
const dialogFormVisible = ref(false)
const data = reactive({
  searchForm: {
    supplierName: '',
  },
  form: {
    supplierId: null,
    supplierName: '',
    productId: null,
    productName: '',
    userId: userStore.userId,
    nickName: '',
    model: '',
    productModelId: null,
    unit: '',
    productrecordId: null,
    taxInclusiveUnitPrice: '',
    taxInclusiveTotalPrice: '',
    taxRate: '',
    taxExclusiveTotalPrice: '',
    inboundTime: '',
    inboundBatch: '',
    inboundQuantity: ''
  },
  rules: {
    productId: [
      { required: true, message: "请选择产品大类", trigger: "change" }
    ],
    productModelId: [
      { required: true, message: "请选择规格型号", trigger: "change" }
    ],
    supplierId: [
      { required: true, message: "请选择供应商", trigger: "change" }
    ],
    inboundQuantity: [
      { required: true, message: "请输入入库数量", trigger: "blur" }
    ],
    userId: [
      { required: true, message: "请选择入库人", trigger: "change" }
    ],
    unit: [{ required: true, message: "请输入", trigger: "blur" }],
    inboundTime: [{ required: true, message: "请选择", trigger: "change" }],
    inboundBatch: [{ required: true, message: "请输入", trigger: "blur" }],
    taxInclusiveUnitPrice: [{ required: true, message: "请输入", trigger: "blur" }],
    taxRate: [{ required: true, message: "请输入", trigger: "blur" }],
  }
})
const { searchForm, form, rules } = toRefs(data)

// 查询列表
/** 搜索按钮操作 */
const handleQuery = () => {
  page.current = 1
  getList()
}
const paginationChange = (obj) => {
  page.current = obj.page;
  page.size = obj.limit;
  getList()
}
const getList = () => {
  tableLoading.value = true
  getStockInPage({ ...searchForm.value, ...page }).then(res => {
    tableLoading.value = false
    tableData.value = res.data.records
    ttotal.value = res.data.total
  }).catch(() => {
    tableLoading.value = false
  })
}

// 正确的产品列表加载方法
const loadProductList = async () => {
  loadingProducts.value = true;
  try {
    const res = await productTreeList();

    // 扁平化处理产品树
    const flattenProducts = (nodes, parentId = null) => {
      let result = [];
      nodes.forEach(node => {
        // 只添加有id和label的节点
        if (node.id && node.label) {
          result.push({
            id: node.id,
            productName: node.label,
            parentId: parentId
          });
        }
        // 递归处理子节点
        if (node.children) {
          result = result.concat(flattenProducts(node.children, node.id));
        }
      });
      return result;
    };

    productList.value = flattenProducts(res.data);
    console.log('加载的产品列表:', productList.value);
  } catch (error) {
    console.error('加载产品列表失败:', error);
    proxy.$modal.msgError('加载产品列表失败');
  } finally {
    loadingProducts.value = false;
  }
  return null; // 没有找到节点，返回null
};
// 表格选择数据
const handleSelectionChange = (selection) => {

// 加载规格型号列表
const loadProductModels = async (productId) => {
  if (!productId) {
    productModelList.value = [];
    return;
  }

  try {
    const res = await modelListPage({
      id: productId,
      current: 1,
      size: 100
    });

    productModelList.value = res.data.records.map(item => ({
      id: item.id,
      model: item.model,
      unit: item.unit
    }));

  } catch (error) {
    console.error('加载规格型号失败:', error);
    proxy.$modal.msgError('加载规格型号失败');
  }
};

// 产品选择变化
const handleProductChange = (productId) => {
  form.value.productModelId = null
  form.value.productrecordId = null
  resetPurchaseFields()
  if (productId) {
    loadProductModels(productId)
  } else {
    productModelList.value = []
  }
}

// 规格型号选择变化
const handleModelChange = async (modelId) => {
  if (!form.value.productId || !modelId) return

  try {
    // 根据产品ID和型号ID查询采购记录
    const res = await getPurchaseRecords({
      productId: form.value.productId,
      productModelId: form.value.productModelId,
      type: 2 // 假设2表示已批准的采购记录
    })

    purchaseRecords.value = res.data.records

    if (purchaseRecords.value.length > 0) {
      // 默认取第一条采购记录
      const record = purchaseRecords.value[0]
      fillFormFromPurchaseRecord(record)
    }
  } catch (error) {
    console.error('查询采购记录失败:', error)
    proxy.$modal.msgError('查询采购记录失败')
  }
}

// 从采购记录填充表单
const fillFormFromPurchaseRecord = (record) => {
  form.value.productrecordId = record.id
  form.value.taxInclusiveUnitPrice = record.taxInclusiveUnitPrice
  form.value.taxRate = record.taxRate
  form.value.taxInclusiveTotalPrice = record.taxInclusiveTotalPrice
  form.value.taxExclusiveTotalPrice = record.taxExclusiveTotalPrice
}

// 重置采购相关字段
const resetPurchaseFields = () => {
  form.value.productrecordId = null
  form.value.unit = ''
  form.value.taxInclusiveUnitPrice = null
  form.value.taxRate = null
  form.value.taxInclusiveTotalPrice = null
  form.value.taxExclusiveTotalPrice = null
}

const loadUserList = async () => {
  try {
    const res = await userListNoPage()
    userList.value = res.data
  } catch (error) {
    console.error('加载用户列表失败:', error)
  }
}

const loadSupplierList = async () => {
  try {
    const res = await listSupplier()
    supplierLists.value = res.data.records
  } catch (error) {
    console.error('加载供应商列表失败:', error)
  }
}

// 打开弹框
const openForm = async (type, row) => {
  operationType.value = type
  dialogFormVisible.value = true
  // 加载必要数据
  await Promise.all([
    loadUserList(),
    loadSupplierList(),
    loadProductList()
  ])

  if (type === 'add') {
    // 新增时重置表单
    form.value = {
      userId: userStore.userId // 设置默认入库人为当前用户
    }
  } else {
    // 编辑时填充表单数据
    form.value = { ...row ,
      supplierId: row.supplierId,
      productId: row.productId,
      productModelId: row.productModelId,
      userId: row.userId
    }
    if (row.productId) {
      await loadProductModels(row.productId)
    }
  }
}

// 提交表单
const submitForm = async () => {
  try {
    await proxy.$refs.formRef.validate()

    if (!form.value.productrecordId) {
      proxy.$modal.msgError('请选择产品大类和规格型号以获取采购记录')
      return
    }

    const submitData = {
      productrecordId: form.value.productrecordId,
      supplierId: form.value.supplierId,
      inboundQuantity: form.value.inboundQuantity,
      userId: form.value.userId,
    }

    const apiMethod = operationType.value === 'add' ? addSutockIn : updateStockIn
    await apiMethod(submitData)

    proxy.$modal.msgSuccess(operationType.value === 'add' ? '新增成功' : '修改成功')
    closeDia()
    getList()
  } catch (error) {
    console.error('提交失败:', error)
    if (!error.errors) {
      proxy.$modal.msgError('操作失败，请重试')
    }
  }
}
// 关闭弹框
const closeDia = () => {
  proxy.$refs.formRef.resetFields()
  dialogFormVisible.value = false
}
// 表格选择数据
const handleSelectionChange = (selection) => {
  // 过滤掉子数据
  selectedRows.value = selection.filter(item => item.id);
  console.log('selection', selectedRows.value)
}
const expandedRowKeys = ref([])

// 主表合计方法
const summarizeMainTable = (param) => {
  return proxy.summarizeTable(param, ['contractAmount', 'taxInclusiveTotalPrice', 'taxExclusiveTotalPrice']);
};

// 导出
const handleOut = () => {
  ElMessageBox.confirm(
    '选中的内容将被导出，是否确认导出？',
    '导出', {
    confirmButtonText: '确认',
    cancelButtonText: '取消',
    type: 'warning',
  }
  ).then(() => {
    proxy.download("/stockin/export", {}, '入库台账.xlsx')
  }).catch(() => {
    proxy.$modal.msg("已取消")
  })
}
// 删除
const handleDelete = () => {
  let ids = []
  if (selectedRows.value.length > 0) {
    ids = selectedRows.value.map(item => item.id);
  } else {
    proxy.$modal.msgWarning('请选择数据')
    return
  }
  ElMessageBox.confirm(
    '选中的内容将被删除，是否确认删除？',
    '导出', {
    confirmButtonText: '确认',
    cancelButtonText: '取消',
    type: 'warning',
  }
  ).then(() => {
    delStockIn(ids).then(res => {
      proxy.$modal.msgSuccess("删除成功")
      getList()
    })
  }).catch(() => {
    proxy.$modal.msg("已取消")
  })
}
// 获取当前日期并格式化为 YYYY-MM-DD
function getCurrentDate() {
  const today = new Date();
  const year = today.getFullYear();
  const month = String(today.getMonth() + 1).padStart(2, '0'); // 月份从0开始
  const day = String(today.getDate()).padStart(2, '0');
  return `${year}-${month}-${day}`;
}

const productSelectedRows = ref([])
const productSelected = (selectedRows) => {
  productSelectedRows.value = selectedRows;
};

const formattedNumber = (row, column, cellValue) => {
  return parseFloat(cellValue).toFixed(2);
};
onMounted(() => {
  getList()
})
</script>

<style scoped lang="scss"></style>
